;ͻ
;Simple example musicroutine for Rant #7                                      
;http://covertbitops.cjb.net                                                  
;                                                                             
;Features of this musicroutine:                                               
;* Note data only (no sequence data)                                          
;* Note data contains notes, rests, continues + change instrument and         
;  jump commands                                                              
;* Instrument has waveform,AD,SR,pulsewidth and pulsewidth speed values       
;* The only effect is pulsewidth modulation                                   
;                                                                             
;This routine doesn't try to be feature-rich, or well optimized, it just      
;tries to be as easy an example as possible.                                  
;ͼ

                processor 6502
                org 2049

;ͻ
;The main program                                                             
;ͼ

sys:            dc.b $0b,$08            ;Address of next instruction
                dc.b $0a,$00            ;Line number(10)
                dc.b $9e                ;SYS-token
                dc.b $32,$30,$36,$31    ;2061 as ASCII
                dc.b $00
                dc.b $00,$00            ;Instruction address 0 terminates
                                        ;the basic program

start:          lda #$00                ;Initialize the playback of the
                jsr $1000               ;first tune
                sei
                lda #<raster            ;Set interrupt vector
                sta $0314
                lda #>raster
                sta $0315
                lda #50                 ;Set low bits of raster
                sta $d012               ;position
                lda $d011
                and #$7f                ;Set high bit of raster
                sta $d011               ;position (0)
                lda #$7f                ;Set timer interrupt off
                sta $dc0d
                lda #$01                ;Set raster interrupt on
                sta $d01a
                lda $dc0d               ;Acknowledge timer interrupt
                cli
                rts

raster:         nop
                nop
                nop
                nop
                nop
                nop
                nop
                nop
                nop
                inc $d020
                jsr $1003               ;Play one frame of music
                dec $d020
                dec $d019
                jmp $ea31

;ͻ
;The musicroutine code itself                                                 
;ͼ

                org $1000

temp1           = $fb                   ;Two zeropage temp variables
temp2           = $fc

C0              = $00                   ;Definitions for note data.
CIS0            = $01                   ;Note numbers are always followed
D0              = $02                   ;by the note duration in frames
DIS0            = $03
E0              = $04
F0              = $05
FIS0            = $06
G0              = $07
GIS0            = $08
A0              = $09
B0              = $0a
H0              = $0b
C1              = $0c
CIS1            = $0d
D1              = $0e
DIS1            = $0f
E1              = $10
F1              = $11
FIS1            = $12
G1              = $13
GIS1            = $14
A1              = $15
B1              = $16
H1              = $17
C2              = $18
CIS2            = $19
D2              = $1a
DIS2            = $1b
E2              = $1c
F2              = $1d
FIS2            = $1e
G2              = $1f
GIS2            = $20
A2              = $21
B2              = $22
H2              = $23
C3              = $24
CIS3            = $25
D3              = $26
DIS3            = $27
E3              = $28
F3              = $29
FIS3            = $2a
G3              = $2b
GIS3            = $2c
A3              = $2d
B3              = $2e
H3              = $2f
C4              = $30
CIS4            = $31
D4              = $32
DIS4            = $33
E4              = $34
F4              = $35
FIS4            = $36
G4              = $37
GIS4            = $38
A4              = $39
B4              = $3a
H4              = $3b
C5              = $3c
CIS5            = $3d
D5              = $3e
DIS5            = $3f
E5              = $40
F5              = $41
FIS5            = $42
G5              = $43
GIS5            = $44
A5              = $45
B5              = $46
H5              = $47
C6              = $48
CIS6            = $49
D6              = $4a
DIS6            = $4b
E6              = $4c
F6              = $4d
FIS6            = $4e
G6              = $4f
GIS6            = $50
A6              = $51
B6              = $52
H6              = $53
C7              = $54
CIS7            = $55
D7              = $56
DIS7            = $57
E7              = $58
F7              = $59
FIS7            = $5a
G7              = $5b
GIS7            = $5c
A7              = $5d
B7              = $5e
H7              = $5f
REST            = $60           ;Rest clears the gatebit
CONTINUE        = $61           ;Continue just continues the note
INSTR           = $80           ;Values $80-$fe are instrument changes
END             = $ff           ;End voice data; followed by a 16-bit jump
                                ;address

                jmp init
                jmp play

init:           sta play_flag+1         ;Songnumber to be played
                rts

play:           ldx #$00                ;X will be the voice index
play_flag:      lda #$00                ;Check the init/play flag
                bmi play_playloop       ;Nonnegative value -> init song
                                        ;(song number is the value)
                                        ;Negative value -> play
                asl
                sta temp1               ;Multiply songnumber by 6 to get
                asl                     ;index to song startaddresstable
                adc temp1
                tay

play_initloop:  lda songtbl,y           ;Copy start address of music data
                sta v_ptrlo,x           ;to voice data pointer (16bit
                lda songtbl+1,y         ;value)
                sta v_ptrhi,x
                iny                     ;Increment index for next voice
                iny
                lda #$02                ;Reset note-duration counter to 2
                sta v_counter,x         ;(new notes will be fetched)
                lda #$08                ;Set testbit in waveform, to reset
                sta v_waveform,x        ;noise waveform if it has been stuck
                inx
                cpx #$03                ;Loop thru all 3 voices
                bcc play_initloop
                lda #$00                ;Reset filter bits - this musicroutine
                sta $d417               ;doesn't use filter
                lda #$0f                ;Reset volume to maximum
                sta $d418
                lda #$ff                ;Set negative value to the playing flag
                sta play_flag+1         ;so that playing can start
                rts

play_initnew:   ldy v_note,x            ;Get notenumber
                cpy #REST               ;If it's a rest or continue, do nothing
                bcs play_initonlycounter
                lda freqtbllo,y         ;Get note's frequency from the
                sta v_freqlo,x          ;frequency table
                lda freqtblhi,y
                sta v_freqhi,x
                ldy v_instr,x           ;Get voice instrument number
                lda i_pulselo,y         ;Copy pulse width initial value,
                sta v_pulselo,x         ;waveform and ADSR from instrument
                lda i_pulsehi,y         ;data to voice variables
                sta v_pulsehi,x
                lda i_waveform,y
                sta v_waveform,x
                lda i_ad,y
                sta v_ad,x
                lda i_sr,y
                sta v_sr,x
play_initonlycounter:
                lda v_counternew,x      ;New value for the note duration
                sta v_counter,x         ;counter
                jmp play_deccounter

play_playloop:  lda v_counter,x         ;Time to init new notes?
                beq play_initnew
                cmp #$02                ;Or fetch new notes?
                beq play_fetchnew
play_pulsemod:  ldy v_instr,x
                lda v_pulselo,x         ;If not, do the pulsewidth modulation,
                clc                     ;decrement note duration counter,
                adc i_pulsespeed,y      ;dump voice variables to SID and
                sta v_pulselo,x         ;go to the next channel.
                lda v_pulsehi,x
                adc #$00
                sta v_pulsehi,x
play_deccounter:dec v_counter,x
                ldy v_regindex,x        ;Get SID register index (0,7,14 for
                lda v_freqlo,x          ;different voices)
                sta $d400,y             ;Frequency
                lda v_freqhi,x
                sta $d401,y
                lda v_pulselo,x
                sta $d402,y             ;Pulse
                lda v_pulsehi,x
                sta $d403,y
                lda v_waveform,x
                sta $d404,y             ;Waveform
                lda v_ad,x
                sta $d405,y             ;Attack/Decay
                lda v_sr,x
                sta $d406,y             ;Sustain/Release
play_nextvoice: inx                     ;Loop until all 3 voices done
                cpx #$03
                bcc play_playloop
                rts

play_fetchnew:  lda v_ptrlo,x           ;Put voice data pointer to zeropage
                sta temp1               ;for indirect access
                lda v_ptrhi,x
                sta temp2
                ldy #$00                ;Y register for indexing voice data
play_fetchloop: lda (temp1),y           ;Get byte from voice data
                bpl play_fetchnote      ;Nonnegative value = note
                iny                     ;Increment index already here to avoid
                                        ;having to do it twice
                cmp #$ff                ;A jump command?
                bcc play_instrchange    ;No, an instrument change
                lda (temp1),y           ;Get lowbyte of jump address
                sta v_ptrlo,x
                iny
                lda (temp1),y           ;Get highbyte of jump address and
                sta v_ptrhi,x           ;go back to fetch loop hoping it isn't
                jmp play_fetchnew       ;an endless loop :)
play_instrchange:
                and #$7f                ;Clear the high bit and we have the
                sta v_instr,x           ;instrument number
                jmp play_fetchloop
play_fetchnote: sta v_note,x            ;Store note number
                iny
                lda (temp1),y           ;Then get note duration
                sta v_counternew,x
                iny
                lda v_note,x            ;Is it a "continue note"-command?
                cmp #CONTINUE           ;If yes, skip the gatebit reset
                beq play_skipgatebit
                lda v_waveform,x
                and #$fe
                sta v_waveform,x
play_skipgatebit:
                tya                     ;Now move the voice data pointer by
                clc                     ;the amount of bytes that were
                adc temp1               ;fetched
                sta v_ptrlo,x
                lda temp2
                adc #$00
                sta v_ptrhi,x
                jmp play_deccounter     ;And go back to the loop (note: pulse
                                        ;modulation is omitted to save some
                                        ;time)

;ͻ
;Voice variables for each voice                                               
;ͼ

v_regindex:     dc.b 0,7,14             ;SID register index
v_freqlo:       dc.b 0,0,0              ;Frequency lowbyte (ghost register)
v_freqhi:       dc.b 0,0,0              ;Frequency highbyte (ghost register)
v_pulselo:      dc.b 0,0,0              ;Pulse lowbyte (ghost register)
v_pulsehi:      dc.b 0,0,0              ;Pulse highbyte (ghost register)
v_waveform:     dc.b 0,0,0              ;Waveform (ghost register)
v_ad:           dc.b 0,0,0              ;Attack/Decay (ghost register)
v_sr:           dc.b 0,0,0              ;Sustain/Release (ghost register)
v_instr:        dc.b 0,0,0              ;Instrument number
v_note:         dc.b 0,0,0              ;Note number
v_counter:      dc.b 0,0,0              ;Note duration counter
v_counternew:   dc.b 0,0,0              ;Duration of new note
v_ptrlo:        dc.b 0,0,0              ;Voice data pointer lowbyte
v_ptrhi:        dc.b 0,0,0              ;Voice data pointer highbyte

;ͻ
;Frequency tables (from GoatTracker playroutines)                             
;ͼ

freqtblhi:      dc.b $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$02
                dc.b $02,$02,$02,$02,$02,$02,$03,$03,$03,$03,$03,$04
                dc.b $04,$04,$04,$05,$05,$05,$06,$06,$06,$07,$07,$08
                dc.b $08,$09,$09,$0a,$0a,$0b,$0c,$0c,$0d,$0e,$0f,$10
                dc.b $11,$12,$13,$14,$15,$17,$18,$19,$1b,$1d,$1e,$20
                dc.b $22,$24,$26,$29,$2b,$2e,$30,$33,$36,$3a,$3d,$41
                dc.b $45,$49,$4d,$52,$57,$5c,$61,$67,$6d,$74,$7b,$82
                dc.b $8a,$92,$9b,$a4,$ae,$b8,$c3,$cf,$db,$e8,$f6,$ff

freqtbllo:      dc.b $15,$25,$36,$49,$5c,$71,$87,$9f,$b7,$d1,$ed,$0a
                dc.b $2a,$4a,$6d,$92,$b9,$e3,$0f,$3e,$6f,$a3,$db,$15
                dc.b $54,$95,$db,$25,$73,$c7,$1e,$7c,$de,$47,$b6,$2b
                dc.b $a8,$2b,$b7,$4b,$e7,$8e,$3d,$f8,$bd,$8e,$6c,$57
                dc.b $50,$57,$6e,$96,$cf,$1c,$7b,$f0,$7b,$1d,$d8,$ae
                dc.b $a0,$af,$dd,$2d,$9f,$38,$f7,$e0,$f6,$3b,$b1,$5d
                dc.b $40,$5e,$bb,$5a,$3f,$70,$ef,$c1,$ed,$76,$63,$ba
                dc.b $80,$bc,$76,$b4,$7f,$e0,$de,$83,$da,$ed,$c7,$ff

;ͻ
;Instrument data (each value sequentially for each instrument)                
;ͼ

i_pulselo:      dc.b $00,$00,$00        ;Pulse width lowbyte
i_pulsehi:      dc.b $02,$00,$00        ;Pulse width highbyte
i_pulsespeed:   dc.b $20,$00,$00        ;Pulse width speed
i_ad:           dc.b $09,$58,$0a        ;Attack/Decay
i_sr:           dc.b $00,$aa,$00        ;Sustain/Release
i_waveform:     dc.b $41,$21,$11        ;Waveform (must have gatebit on)

;ͻ
;Song table (address of voice data start for each voice)                      
;ͼ

songtbl:        dc.w song0voice1
                dc.w song0voice2
                dc.w song0voice3
                ;More songs could be added here

;ͻ
;Voice 1 data                                                                 
;ͼ

song0voice1:    dc.b INSTR+0            ;Set instrument 0
                dc.b A2,14              ;Note/duration pairs
                dc.b A2,14
                dc.b A2,14
                dc.b H2,7
                dc.b C3,7
                dc.b D3,14
                dc.b D3,14
                dc.b D3,14
                dc.b C3,14
                dc.b G2,14
                dc.b G2,14
                dc.b G2,14
                dc.b A2,7
                dc.b H2,7
                dc.b C3,14
                dc.b C3,14
                dc.b C3,14
                dc.b H2,14
                dc.b F2,14
                dc.b F2,14
                dc.b F2,14
                dc.b G2,7
                dc.b A2,7
                dc.b H2,14
                dc.b H2,14
                dc.b H2,14
                dc.b A2,14
                dc.b E2,14
                dc.b E2,14
                dc.b E2,14
                dc.b E2,14
                dc.b E2,7
                dc.b F2,7
                dc.b E2,7
                dc.b D2,7
                dc.b E2,7
                dc.b F2,7
                dc.b G2,7
                dc.b H2,7
                dc.b END                        ;End command, followed by
                dc.w song0voice1                ;restart address

;ͻ
;Voice 2 data                                                                 
;ͼ

song0voice2:    dc.b INSTR+1                    ;Set instrument 1
                dc.b C4,7
                dc.b REST,7                     ;Rest also has a duration
                dc.b C4,7
                dc.b REST,7
                dc.b C4,7
                dc.b REST,7
                dc.b C4,7
                dc.b REST,7
                dc.b D4,7
                dc.b C4,7
                dc.b H3,7
                dc.b A3,7
                dc.b G3,7
                dc.b A3,7
                dc.b H3,7
                dc.b C4,7
                dc.b H3,7
                dc.b REST,7
                dc.b H3,7
                dc.b REST,7
                dc.b H3,7
                dc.b REST,7
                dc.b H3,7
                dc.b REST,7
                dc.b C4,7
                dc.b H3,7
                dc.b A3,7
                dc.b G3,7
                dc.b A3,7
                dc.b H3,7
                dc.b C4,7
                dc.b D4,7
                dc.b A3,7
                dc.b REST,7
                dc.b A3,7
                dc.b REST,7
                dc.b A3,7
                dc.b REST,7
                dc.b A3,7
                dc.b REST,7
                dc.b D4,7
                dc.b C4,7
                dc.b H3,7
                dc.b A3,7
                dc.b GIS3,7
                dc.b FIS3,7
                dc.b GIS3,7
                dc.b A3,7
                dc.b C4,42
                dc.b H3,7
                dc.b A3,7
                dc.b H3,42
                dc.b REST,14
                dc.b E4,7
                dc.b REST,7
                dc.b C4,7
                dc.b H3,7
                dc.b C4,7
                dc.b REST,7
                dc.b A4,7
                dc.b REST,7
                dc.b A4,7
                dc.b G4,7
                dc.b F4,7
                dc.b E4,7
                dc.b D4,7
                dc.b E4,7
                dc.b F4,7
                dc.b G4,7
                dc.b D4,7
                dc.b REST,7
                dc.b G3,7
                dc.b FIS3,7
                dc.b G3,7
                dc.b REST,7
                dc.b F4,7
                dc.b REST,7
                dc.b F4,7
                dc.b E4,7
                dc.b D4,7
                dc.b E4,7
                dc.b C4,7
                dc.b REST,7
                dc.b E4,7
                dc.b REST,7
                dc.b C4,7
                dc.b REST,7
                dc.b F3,7
                dc.b REST,7
                dc.b A3,7
                dc.b REST,7
                dc.b C4,7
                dc.b REST,7
                dc.b DIS4,7
                dc.b REST,7
                dc.b A3,7
                dc.b REST,7
                dc.b C4,7
                dc.b REST,7
                dc.b DIS4,7
                dc.b REST,7
                dc.b E4,28
                dc.b FIS4,28
                dc.b GIS4,42
                dc.b REST,14
                dc.b END
                dc.w song0voice2

;ͻ
;Voice 3 data                                                                 
;ͼ

song0voice3:    dc.b INSTR+2                    ;Set instrument 2
                dc.b A4,42
                dc.b H4,7
                dc.b C5,7
                dc.b D5,56
                dc.b G4,42
                dc.b A4,7
                dc.b H4,7
                dc.b C5,56
                dc.b F4,42
                dc.b G4,7
                dc.b A4,7
                dc.b H4,56
                dc.b E4,35
                dc.b F4,7
                dc.b E4,7
                dc.b D4,7
                dc.b E4,56
                dc.b A4,42
                dc.b H4,7
                dc.b C5,7
                dc.b D5,56
                dc.b G4,42
                dc.b A4,7
                dc.b H4,7
                dc.b C5,56
                dc.b F4,42
                dc.b G4,7
                dc.b A4,7
                dc.b H4,56
                dc.b E4,35
                dc.b A4,7
                dc.b H4,7
                dc.b C5,7
                dc.b D5,56
                dc.b END
                dc.w song0voice3
